#!/usr/bin/env python
#####################################################################################
#
# Copyright (c) Members of the EGEE Collaboration. 2004.
# See http://eu-egee.org/partners/ for details on the copyright holders.
# For license conditions see the license file or http://eu-egee.org/license.html
#
#####################################################################################
# glite-ce-wss-config v. 2.4.0
#
# Post-installation script for configuring the gLite Computing Element Service
# Alberto Di Meglio  < alberto.di.meglio@cern.ch >
# Mario Reale  < mario.reale@cern.ch >
# Leanne Guy  < leanne.guy@cern.ch >
#
# Version info: $Id: glite-ce-wss-config.py,v 1.6 2006/03/01 11:11:19 flammer Exp $
#
# Usage: python glite-ce-config [-c|-v|-h|--help]
#        -c                                    print configuration
#        -v                                    print version
#        -h,--help                             print usage info
#        --configure                           configure the service
#        --start                               start the service
#        --stop                                stop the service
#        --status                              print service status (0 = running, 1 = idle)
#        --siteconfig=<site config URL>        use remote site configuration file from URL
#
# Return codes: 0 - Ok
#               1 - Configuration failed
#
#####################################################################################

import os,string,socket,pwd,grp,time,re
import sys
import getopt
import commands
import _xmlplus
import _xmlplus.xpath as xpath
import xml.dom.minidom
from xml.dom.minidom import Node
from xml.dom.ext import PrettyPrint

from  gLiteInstallerLib import gLib
from condor import Condor
from globus import Globus
from tomcat5 import Secure_tomcat5 as Tomcat5
from gliteRgmaServicetool import gliteRgmaServicetool

# Set global variables here 
global params                        # all config values from the XML file
global condor                        # the one and only condor service instance
global globus                        # the one and only globus service instance
global tomcat                        # the one and only globus service instance
global rgmaServicetool

class glite_ce:

    def __init__(self):
        self.verbose = 0
        self.iptables_config = 1
        self.version = "2.4.0"
        self.name = "glite-ce-wss"
        self.friendly_name = "gLite Computing Element with Workspace Service support"
        params['module.version'] = self.version
        
    #-------------------------------------------------------------------------------
    # Banner 
    #-------------------------------------------------------------------------------

    def banner(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "Configuring the %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"
    
    #-------------------------------------------------------------------------------
    # Copyright 
    #-------------------------------------------------------------------------------

    def copyright(self):

        print '\nCopyright (c) Members of the EGEE Collaboration. 2004'
        print 'See http://eu-egee.org/partners/ for details on the copyright holders'
        print 'For license conditions see the license file or http://eu-egee.org/license.html'

    #-------------------------------------------------------------------------------
    # Version
    #-------------------------------------------------------------------------------

    def showVersion(self):

        print '\n%s-config  v. %s\n' % (self.name,self.version)
    
    #-------------------------------------------------------------------------------
    # Usage
    #-------------------------------------------------------------------------------

    def usage(self,msg = ""):

        if msg:
            print "\n%s" % (msg)
        
        self.copyright()
        self.showVersion()
    
        print """Usage: \n
Edit the configuration file %s.cfg.xml in
%s/etc.config/templates\n
save it as %s/etc/config/%s.cfg.xml
and run the script as follows\n 
python %s-config [OPTION...]""" % (self.name, os.environ['GLITE_LOCATION'], \
        os.environ['GLITE_LOCATION'], self.name, self.name)

        print '    -n, --noiptables    disables the automatic iptables configuration'
        print '    -c, --checkconf     print the service configuration'
        print '    -v, --version       print the version of the configuration script'
        print '    -h, --help          print this usage information'
        print '    --configure         configure the service'
        print '    --start             start the service'
        print '    --stop              stop the service'
        print '    --status            print service status (0 = running, 1 = idle)'
        print '    --siteconfig=<URL>  use site configuration file at URL'
        print '\n'

    #-------------------------------------------------------------------------------
    # All the configuration code goes here
    #-------------------------------------------------------------------------------

    def configure(self):
        
        # Set default error level
        error_level = 0
        
        #----------------------------------------
        # Parameter checks
        #----------------------------------------
        
        if not params.has_key('cemon.lrms'):
            params['cemon.lrms'] = "pbs"
        else:
            if params['cemon.lrms'] == "":
                params['cemon.lrms'] = "pbs"
                
        if not params['cemon.lrms'] in ['pbs','lsf','condor']:
            glib.printErrorMessage("Unrecognised LRMS type. Cannot continue")
            return 1
            
        #-------------------------------------------------------------------------------
        # Update the $GLITE_LOCATION/etc/glite-mkgridmap.conf file if required
        #-------------------------------------------------------------------------------
        
        if params.has_key('install.mkgridmap.cron'):
            if params['install.mkgridmap.cron'] == "true":
                
                mkgc_file = '%s/etc/glite-mkgridmap.conf' % os.environ['GLITE_LOCATION']
                mkgc_file1 = '%s/etc/glite-mkgridmap.conf.1' % os.environ['GLITE_LOCATION']
            
                if self.verbose:
                    print "Updating the %s file..." % mkgc_file
            
                if os.path.exists(mkgc_file1):
                    os.remove(mkgc_file1)
                if os.path.exists(mkgc_file):
                    os.system("/bin/cp %s %s" % (mkgc_file, mkgc_file1))
                else:
                    os.system("/bin/touch %s" % mkgc_file)
        
                vp1 = params['voms.voname']
                vp2 = params['voms.vomsnode']
                vp3 = params['pool.account.basename'] 
         
                for idx in xrange(len(params['voms.voname'])):
        
                    if self.verbose:
                        print "Creating\Verifying group entry for VO: %s..." % vp1[idx]
        
                    groupentry = "group vomss://%s:8443/voms/%s .%s" \
                        % (vp2[idx], vp1[idx], vp3[idx])
                        
                    file = open(mkgc_file, 'r')
                    for line in file:
                        if line.find(groupentry) != -1:
                            break
                    else:
                        file.close()
                        file = open(mkgc_file, 'a')
                        file.write("\n" + groupentry + "\n")  
        
                    file.close()
                
                if not os.path.exists(mkgc_file):
                    print "Writing the %s file                                    " % mkgc_file
                    glib.printFailedMessage()
                    glib.printErrorMessage("Failed to create %s" % mkgc_file)
                    return 1
            
                if self.verbose:
                    print "Writing the %s file                                    " % mkgc_file,
                    glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Installs the Security Utilities
        #--------------------------------------------------------
        
        if os.system("python %s/glite-security-utils-config.py --subservice" % glib.getScriptPath()):
            print "\nInstalling gLite Security Utilities               ",
            glib.printFailedMessage()
            return 1

        print "\nInstalling gLite Security Utilities                        ",
        glib.printOkMessage()
            
        #-------------------------------------------------------------------------------
        # Verify the JAVA installation and set it in /etc/java.conf
        #-------------------------------------------------------------------------------
        
        if glib.checkJava() != 0:
            return 1
        
        #-------------------------------------------------------------------------------
        # Configure Tomcat
        #-------------------------------------------------------------------------------
        
        print "\nConfiguring Tomcat ...                                  "
        
        # stop tomcat first
        tomcat.stop()
        
        # configure tomcat
        errorcode = tomcat.configure()
        if errorcode != 0:
            print "Configuring Tomcat                                             ",
            glib.printFailedMessage()
            return 1
             
        print "Configuring Tomcat                                           ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # VO list notice
        #-------------------------------------------------------------------------------
    
        print "\nThis CE will be configured to serve the following VOs:"
        for vo in params['voms.voname']:
            print "=> %s" % vo
            
        #-------------------------------------------------------------------------------
        # Create N mapped accounts for each VO after having added the 'mapped' group
        #-------------------------------------------------------------------------------
    
        if params.has_key('host.gridmap.dir'):
            gridmap_dir = params['host.gridmap.dir']
        else:
            gridmap_dir = '/etc/grid-security/gridmapdir'
        glib.check_dir(gridmap_dir)

        if not params.has_key('account.discovery'):
            params['account.discovery'] = 'false'
            
        if params['account.discovery'] == 'false':
            
            if params.has_key('pool.account.group') and \
                params.has_key('pool.account.basename') and \
                params.has_key('pool.account.number'):
                    
                pgroup = params['pool.account.group']
                pbase = params['pool.account.basename']
                pnumber = params['pool.account.number']
            
                if params.has_key('pool.lsfgid'):
                    lsfgid = params['pool.lsfgid']
                    for idx in xrange(len(pgroup)):
                        if lsfgid[idx] == "":
                            lsfgid[idx] = '0'
                else:
                    lsfgid = []
                    for item in pgroup:
                        lsfgid.append('0')
                    
                if self.verbose:
                    print "\nCreating/Verifying the pool accounts ..."
            
                for idx in xrange(len(params['voms.voname'])):
    
                    glib.printInfoMessage("Pool accounts for VO: %s" % params['voms.voname'][idx])
    
                    # Create group if doesn't exist, scripts terminates
                    # if lsfgid is specified and group already exist with different GID
                    if pgroup[idx] == "":
                        pgroup[idx] = pbase[idx]
                    if self.verbose:
                        print "==> Verifying group %s " % pgroup[idx]
                    if int(lsfgid[idx]):
                        gid = glib.add_group(pgroup[idx],lsfgid[idx])
                    else:
                        gid = glib.add_group(pgroup[idx])
                    if self.verbose:
                        glib.printInfoMessage("Group %s has been added or verified" % pgroup[idx])
            
                    # Create pool accounts in given group, this also sets the environment for the users
                    glib.create_pool(pbase[idx],int(pnumber[idx]),pgroup[idx])
        
        else:
            
            # Locate pool accounts 
            l = pwd.getpwall()
            for idx in xrange(len(params['voms.voname'])):

                glib.printInfoMessage("Discovering pool accounts for VO: %s" % params['voms.voname'][idx])
                discoacc = []
                for acc in l:
                    if re.match('%s[0-9]+$' % params['pool.account.basename'][idx],acc[0]):
                        discoacc.append(acc[0])
                        
                for acc in discoacc:
                    glib.setUserEnv(acc)
                    # Touch the gridmap mapping files if they don't exist
                    if not os.path.exists("%s/%s" % (gridmap_dir,acc)):
                        os.system("/bin/touch %s/%s" % (gridmap_dir,acc))

            
        print "Creating/Verifying the pool accounts                     ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Create  the $GLITE_LOCATION/grid-services directory
        #-------------------------------------------------------------------------------
        
        if self.verbose:
            print "Creating the %s/etc/grid-services directory..." % os.environ['GLITE_LOCATION']
    
        path = '%s/etc/grid-services' % os.environ['GLITE_LOCATION']
        if not os.path.exists(path):
            os.mkdir(path,0755)
            os.chown(path,0,0)
    
        if self.verbose:
            print "Creating the %s/etc/grid-services directory     " % os.environ['GLITE_LOCATION'],
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create the gatekeeper config file $GLITE_LOCATION/etc/gatekeeper.conf
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Creating the %s/etc/gatekeeper.conf file..." % os.environ['GLITE_LOCATION']
        
        path = "%s/etc/gatekeeper.conf" % os.environ['GLITE_LOCATION']
        path1 = "%s/etc/gatekeeper.conf.1" % os.environ['GLITE_LOCATION']
        if os.path.exists(path1):
            os.remove(path1)
        if os.path.exists(path):
            os.rename(path,path1)
        file = open(path, 'w')
        file.write("-x509_cert_dir %s\n" % params['ca.certificates.dir'])
        file.write("-x509_user_cert %s\n" % params['host.certificate.file'])
        file.write("-x509_user_key %s\n" % params['host.key.file'])
        file.write("-gridmap %s\n" % params['host.gridmapfile'])
        file.write("-home %s\n" % os.environ['GLITE_LOCATION'])
        file.write("-e libexec\n")
        file.write("-port %s\n" % params['gatekeeper.port'])
        file.write("-grid_services %s/etc/grid-services\n" % os.environ['GLITE_LOCATION'])
        file.write('-logfile "%s/gatekeeper.log"\n' % os.environ['GLITE_LOCATION_LOG'])
        file.write("-lcas_db_file lcas.db\n")
        file.write("-lcas_etc_dir %s/etc/lcas/\n" % os.environ['GLITE_LOCATION'])
        file.write("-lcasmod_dir %s/lib/\n" % os.environ['GLITE_LOCATION'])
        file.write("-lcmaps_db_file lcmaps.db\n")
        file.write("-lcmaps_etc_dir %s/etc/lcmaps\n" % os.environ['GLITE_LOCATION'])
        file.write("-lcmapsmod_dir %s/lib/\n" % os.environ['GLITE_LOCATION'])
        file.close()
        
        if self.verbose:
            print "Creating the %s/etc/gatekeeper.conf file         " % os.environ['GLITE_LOCATION'],
            glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Create the jobmanager-fork file $GLITE_LOCATION/etc/grid-services/jobmanager-fork:
        #-------------------------------------------------------------------------------

        if self.verbose:
            print "Creating the %s/etc/grid-services/jobmanager-fork file and link..." % os.environ['GLITE_LOCATION']
    
        jobmanager_fork = "%s/etc/grid-services/jobmanager-fork" % os.environ['GLITE_LOCATION']
        jobmanager_fork1 = "%s/etc/grid-services/jobmanager-fork.1" % os.environ['GLITE_LOCATION']
        if os.path.exists(jobmanager_fork1):
            os.remove(jobmanager_fork1)
        if os.path.exists(jobmanager_fork):
            os.rename(jobmanager_fork,jobmanager_fork1)
        file = open(jobmanager_fork, 'w')
        file.write("stderr_log,local_cred - %s/libexec/globus-job-manager \
globus-job-manager -conf %s/etc/globus-job-manager.conf -type fork -rdn jobmanager-fork \
-machine-type unknown -publish-jobs \n" % ( os.environ['GLOBUS_LOCATION'], os.environ['GLITE_LOCATION'] ))
        file.close()
    
        if not os.path.exists(jobmanager_fork):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s \n" % jobmanager_fork)
            return 1
    
        jobmanager_link = '%s/etc/grid-services/jobmanager' % os.environ['GLITE_LOCATION']
    
        if os.path.exists(jobmanager_link):
            os.remove(jobmanager_link)
    
        os.symlink(jobmanager_fork, jobmanager_link)
    
        if not os.path.exists(jobmanager_link):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create symlink s \n" % jobmanager_link)
            return 1
    
        if self.verbose:
            print "Creating the %s/etc/grid-services/jobmanager-fork file and link    " % os.environ['GLITE_LOCATION'],
            glib.printOkMessage()
           
        #-------------------------------------------------------------------------------
        # Create the  globus job manager config file $GLITE_LOCATION/etc/globus-job-manager.conf :  
        #-------------------------------------------------------------------------------
    
        globus_jobmanager_conf = '%s/etc/globus-job-manager.conf' % os.environ['GLITE_LOCATION']
        globus_jobmanager_conf1 = '%s/etc/globus-job-manager.conf.1' % os.environ['GLITE_LOCATION']
    
        if self.verbose:
            print "Creating the globus job manager config file %s..." % globus_jobmanager_conf
    
        if os.path.exists(globus_jobmanager_conf1):
            os.remove(globus_jobmanager_conf1)
        if os.path.exists(globus_jobmanager_conf):
            os.rename(globus_jobmanager_conf,globus_jobmanager_conf1)
    
        osname = os.uname()[2]
        if params.has_key('globus.osversion'):
            if params['globus.osversion'] != "":
                osname = params['globus.osversion']
        
        # Hostname
        hn=socket.gethostname()  

        # Host DN subject
        if params.has_key('globus.hostdn'):
            if params['globus.hostdn'] != "":
                gdn = params['globus.hostdn']
            else:
                try:
                    if os.path.exists(params['host.certificate.file']):
                        gdn = glib.get_certificate_subject(params['host.certificate.file'])
                    else:
                        glib.printFailedMessage()
                        glib.printErrorMessage("Could not find a valid host certificate at %s" % params['host.certificate.file'])
                        return 1
                except AttributeError, e:
                    print e
                    return 1
        else:
            try:
                if os.path.exists(params['host.certificate.file']):
                    gdn = glib.get_certificate_subject(params['host.certificate.file'])
                else:
                    glib.printFailedMessage()
                    glib.printErrorMessage("Could not find a valid host certificate at %s" % params['host.certificate.file'])
                    return 1
            except AttributeError, e:
                print e
                return 1

        file = open(globus_jobmanager_conf, 'w')
        file.write('-home "%s" \n' % os.environ['GLOBUS_LOCATION'])
        file.write("-globus-gatekeeper-host %s\n" % hn )
        file.write("-globus-gatekeeper-port %s\n" % params['gatekeeper.port'])
        file.write('-globus-gatekeeper-subject "%s"\n' % gdn)
        file.write("-globus-host-cputype %s\n" % os.uname()[4])
        file.write("-globus-host-manufacturer pc\n")
        file.write("-globus-host-osname %s\n"  % os.uname()[0])
        file.write('-globus-host-osversion "%s"\n' % osname)
        file.write("-save-logfile on_error\n")
        file.write("-state-file-dir %s/gram_job_state\n" % os.environ['GLITE_LOCATION_VAR'])
        file.write("-machine-type unknown\n")
        file.write('-globus-tcp-port-range "20000 25000"\n')
        file.close()
        
        if not os.path.exists(globus_jobmanager_conf):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % globus_jobmanager_conf)
            return 1
    
        if self.verbose:
            print "Creating the globus job manager config file %s     " % globus_jobmanager_conf,
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create  the $GLITE_LOCATION_TMP dir and set permissions:
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Creating directory %s and setting permissions..."  % os.environ['GLITE_LOCATION_TMP']
    
        os.system('/bin/mkdir -p %s' % os.environ['GLITE_LOCATION_TMP'])
        if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % os.environ['GLITE_LOCATION_TMP'])
            return 1
        os.system('/bin/chmod 01777 %s' % os.environ['GLITE_LOCATION_TMP'])
    
        if self.verbose:
            print "Creating directory %s and setting permissions    "  % os.environ['GLITE_LOCATION_TMP']
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create  the  $GLITE_LOCATION_VAR/gram_job_state dir and set corresponding permissions:
        #-------------------------------------------------------------------------------
    
        gram_job_state_dir = '%s/gram_job_state' % os.environ['GLITE_LOCATION_VAR']
    
        if self.verbose:
            print "Creating directory %s and setting permissions..." % gram_job_state_dir
    
        os.system('/bin/mkdir -p %s ' % gram_job_state_dir)
        if not os.path.exists(gram_job_state_dir):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % gram_job_state_dir)
            return 1
        os.system('/bin/chmod 01777 %s' %  gram_job_state_dir)
            
        if self.verbose:
            print "Creating directory %s and setting permissions     " % gram_job_state_dir,
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create the $GLITE_LOCATION/etc/lcas/lcas.db file :
        #-------------------------------------------------------------------------------
    
        lcas_db_file = '%s/etc/lcas/lcas.db' % os.environ['GLITE_LOCATION']
        lcas_db_file1 = '%s/etc/lcas/lcas.db.1' % os.environ['GLITE_LOCATION']
    
        if self.verbose:
            print "Creating the %s file..." % lcas_db_file,
     
        if not os.path.exists(os.path.dirname(lcas_db_file)):
            os.makedirs(os.path.dirname(lcas_db_file))
    
        if os.path.exists(lcas_db_file1):
            os.remove(lcas_db_file1)
        if os.path.exists(lcas_db_file):
            os.rename(lcas_db_file,lcas_db_file1)
    
        file = open(lcas_db_file, 'w')
        
        file.write('pluginname=lcas_userban.mod,pluginargs=ban_users.db\n')
        file.write('pluginname=lcas_voms.mod,pluginargs="-vomsdir %s \
-certdir %s \
-authfile %s \
-authformat simple \
-use_user_dn" \n' \
            % (os.environ['X509_VOMS_DIR'], params['ca.certificates.dir'], params['host.gridmapfile']) )
            
        file.close()
        
        if not os.path.exists(lcas_db_file):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % lcas_db_file)
            return 1
    
        if self.verbose:
            print "Creating the %s file                         " % lcas_db_file,
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create an empty banned users  $GLITE_LOCATION/etc/lcas/ban_users.db file
        #-------------------------------------------------------------------------------
    
        banned_users_file = '%s/etc/lcas/ban_users.db' % os.environ['GLITE_LOCATION']
    
        if not os.path.exists(banned_users_file):
            if self.verbose:
                print "Creating an empty banned users file in %s to start with..." % banned_users_file
            os.system('/bin/touch %s' % banned_users_file),
        else:
            if self.verbose:
                print "Banned user file %s already exists  " %  banned_users_file
    
        if not os.path.exists(banned_users_file):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % banned_users_file)
            return 1
    
        #-------------------------------------------------------------------------------
        # Create the gridmap file :
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Updating the grid-mapfile %s..." % params['host.gridmapfile']
        
        # Touch grid-mapfile if it doesn't exist
        if not os.path.exists(params['host.gridmapfile']):
            os.system("/bin/touch %s" % params['host.gridmapfile'])

        # Add pool account entries
        if self.verbose:
            print "Adding pool account entries..."

        for idx in xrange(len(params['voms.voname'])):

            if self.verbose:
                print "Adding/Verifying '/%s/* .%s'" % (params['voms.voname'][idx], params['pool.account.basename'][idx])
            
            file = open(params['host.gridmapfile'], 'r' )
            for line in file.xreadlines():
                if line.find('/%s/* .%s' % (params['voms.voname'][idx], params['pool.account.basename'][idx])) != -1:
                    file.close()
                    break
            else:
                file.close()
                file = open(params['host.gridmapfile'], 'a')
                file.write('\n/%s/* .%s\n' % (params['voms.voname'][idx], params['pool.account.basename'][idx]))
                file.close()

        # Add WMS host entries
        if params.has_key('cemon.wms.host.subject'):

            if self.verbose:
                print "Adding WMS host entries..."
    
            for idx in xrange(len(params['cemon.wms.host.subject'])):
    
                if params['cemon.wms.host.subject'][idx].strip() != "":
                    file = open(params['host.gridmapfile'], 'r' )
                    for line in file.xreadlines():
                        if line.find('"%s"' % params['cemon.wms.host.subject'][idx]) != -1:
                            file.close()
                            break
                    else:
                        file.close()
                        file = open(params['host.gridmapfile'], 'a')
                        file.write('\n"%s"\n' % params['cemon.wms.host.subject'][idx])
                        file.close()

        if self.verbose:
            print "Updating the grid-mapfile %s                " % params['host.gridmapfile'],
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Create the group map file /etc/grid-security/groupmapfile :
        #-------------------------------------------------------------------------------
    
        groupmap_file = "%s" % params['host.groupmapfile']
    
        if self.verbose:
            print "Updating the group mapfile %s..." % groupmap_file
    
        if not os.path.exists(groupmap_file):
            os.system("/bin/touch %s" % groupmap_file)

        for idx in xrange(len(params['voms.voname'])):

            if self.verbose:
                print "Adding/Verifying '/%s/* %s'" % (params['voms.voname'][idx], params['pool.account.group'][idx])
            
            file = open(groupmap_file, 'r' )
            for line in file.xreadlines():
                if line.find('/%s/* %s' % (params['voms.voname'][idx], params['pool.account.group'][idx])) != -1:
                    file.close()
                    break
            else:
                file.close()
                file = open(groupmap_file, 'a')
                file.write('\n/%s/* %s\n' % (params['voms.voname'][idx], params['pool.account.group'][idx]))
                file.close()
        
        if self.verbose:
            print "Updating the group mapfile %s                    " % groupmap_file,
            glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Create the LCMAPS DB file $GLITE_LOCATION/etc/lcmaps/lcmaps.db : 
        #-------------------------------------------------------------------------------
    
        lcmaps_db_file = '%s/etc/lcmaps/lcmaps.db' % os.environ['GLITE_LOCATION']
        lcmaps_db_file1 = '%s/etc/lcmaps/lcmaps.db.1' % os.environ['GLITE_LOCATION']
    
        if self.verbose:
            print "Creating the LCMAPS DB without GSI file %s..." % lcmaps_db_file
    
        if os.path.exists(lcmaps_db_file1):
            os.remove(lcmaps_db_file1)
        if os.path.exists(lcmaps_db_file):
            os.rename(lcmaps_db_file,lcmaps_db_file1)

        if not os.path.exists(os.path.dirname(lcmaps_db_file)):
            os.makedirs(os.path.dirname(lcmaps_db_file))

        file = open(lcmaps_db_file, 'w')
        file.write("# LCMAPS policy file/plugin definition\n")
        file.write("\n")
        file.write("# default path\n")
        file.write('path = %s/lib/modules\n' % os.environ['GLITE_LOCATION'] ) 
        file.write("\n")
        file.write("# Plugin definitions\n")
        file.write('good = "lcmaps_dummy_good.mod"\n')
        file.write('bad = "lcmaps_dummy_bad.mod"\n')
        file.write('localaccount = "lcmaps_localaccount.mod"\n')
        file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'] )
        file.write('poolaccount = "lcmaps_poolaccount.mod"\n')
        file.write('" -override_inconsistency"\n')
        file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'] )
        file.write('" -gridmapdir %s"\n'% params['host.gridmap.dir'])
        file.write('vomslocalgroup = "lcmaps_voms_localgroup.mod"\n')
        file.write('" -groupmapfile %s"\n' % params['host.groupmapfile'])
        file.write('" -mapmin 0"\n')
        file.write('vomspoolaccount = "lcmaps_voms_poolaccount.mod"\n')
        file.write('" -gridmapfile %s"\n' % params['host.gridmapfile'])
        file.write('" -gridmapdir %s"\n' % params['host.gridmap.dir'])
        file.write('" -do_not_use_secondary_gids"\n')
        file.write('posix_enf = "lcmaps_posix_enf.mod"\n')
        file.write('" -maxuid 1"\n')
        file.write('" -maxpgid 1"\n')
        file.write('" -maxsgid 32"\n')
        file.write(' \n')
        file.write('# Policies\n')
        file.write('voms:\n')
        file.write('localaccount -> posix_enf | vomslocalgroup\n')
        file.write('vomslocalgroup -> vomspoolaccount\n')
        file.write('vomspoolaccount -> posix_enf\n')
        file.write('\n')
        file.write('standard:\n')
        file.write('localaccount -> posix_enf | poolaccount\n')
        file.write('poolaccount -> posix_enf\n')
        file.close()
        
        if not os.path.exists(lcmaps_db_file):
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % lcmaps_db_file)
            return 1
    
        if self.verbose:
            print "Creating the LCMAPS DB without GSI file %s              " % lcmaps_db_file,
            glib.printOkMessage()
     
        #-------------------------------------------------------------------------------
        # Create the /etc/grid-security/vomsdir
        #-------------------------------------------------------------------------------
    
        voms_dir = "%s" % os.environ['X509_VOMS_DIR']
        if self.verbose:
            print "Creating %s..." % voms_dir
    
        if not os.path.exists(voms_dir):
            os.makedirs(voms_dir,0755)
    
        if self.verbose:
            print "Creating %s                                 " % voms_dir,
            glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Create the $GLITE_LOCATION/etc/vomses file 
        #-------------------------------------------------------------------------------
        
        vomses_file = '%s/etc/vomses' % os.environ['GLITE_LOCATION']
        vomses_file1 = '%s/etc/vomses.1' % os.environ['GLITE_LOCATION']
    
        if self.verbose:
            print "Writing the %s file..." % vomses_file
    
        if os.path.exists(vomses_file1):
            os.remove(vomses_file1)
        if os.path.exists(vomses_file):
            os.rename(vomses_file,vomses_file1)

        vp1 = params['voms.voname']
        vp2 = params['voms.vomsnode']
        vp3 = params['voms.vomscertsubj'] 
        vp4 = params['voms.vomsport'] 
 
        file = open(vomses_file, 'w')
        
        for idx in xrange(len(params['voms.voname'])):

            if self.verbose:
                print "Creating vomses entry for VO: %s..." % params['voms.voname'][idx]
            
            file.write(' "%s" "%s" "%s" "%s" "%s"\n' % (vp1[idx], vp2[idx], vp4[idx], vp3[idx], vp1[idx] ))  

        file.close()
        
        if not os.path.exists(vomses_file):
            print "Writing the %s file                          " % vomses_file
            glib.printFailedMessage()
            glib.printErrorMessage("Failed to create %s" % vomses_file)
            return 1
    
        if self.verbose:
            print "Writing the %s file                          " % vomses_file,
            glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Globus configuration
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring Globus"
    
        # Configure Globus
        if globus.postInstall() != 0:
            print "Configuring Globus                                           ",
            glib.printFailedMessage()
            return 1

        print "Configuring Globus                                           ",
        glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # Modify condor-c config file
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring Condor-C..."
    
        # Configure Condor C
        if condor.configureCondorC() != 0:
            print "Configuring Condor-C                                         ",
            glib.printFailedMessage()
            return 1
    
        # Set environment for the Condor user
        glib.setUserEnv(params['condor.user'])
        
        print "Configuring Condor-C                                             ",
        glib.printOkMessage()
        
        #-------------------------------------------------------------------------------
        # LRMS Log Parser configuration
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring the BLAHP log parser..."
            
        # Set default values
        if not params.has_key('use.log.parser'):
            params['use.log.parser'] = "false"
        else:
            if params['use.log.parser'] != "true":
                params['use.log.parser'] = "false"
    
        if not params.has_key('log.parser.address'):
            params['log.parser.address'] = "127.0.0.1"
        else:
            if params['log.parser.address'] == "":
                params['log.parser.address'] = "127.0.0.1"

        # Update script files
        for key in ["%s/bin/pbs_submit.sh" % os.environ['GLITE_LOCATION'],
                    "%s/bin/pbs_status.sh" % os.environ['GLITE_LOCATION'],
                    "%s/bin/lsf_submit.sh" % os.environ['GLITE_LOCATION'],
                    "%s/bin/lsf_status.sh" % os.environ['GLITE_LOCATION']]:

            if self.verbose:
                print "Reconfiguring %s script" % key
                
            if params['use.log.parser'] == "false":

                inFile = open(key,'r')
                outFile = open(key + ".new",'w')
                
                for line in inFile:
                    if line.strip().startswith("BLParser"):
                        outFile.write('BLParser=""\n')
                    elif line.strip().startswith("BLPserver"):
                        outFile.write('BLPserver="127.0.0.1"\n')
                    elif line.strip().startswith("BLPport"):
                        outFile.write('BLPport=""\n')
                    else:
                        outFile.write(line)
                        
                outFile.close()
                inFile.close()
                
                os.rename(key + ".new",key)
    
            else:
    
                inFile = open(key,'r')
                outFile = open(key + ".new",'w')
                
                for line in inFile:
                    if line.strip().startswith("BLParser"):
                        outFile.write('BLParser="yes"\n')
                    elif line.strip().startswith("BLPserver"):
                        outFile.write('BLPserver="%s"\n' % params['log.parser.address'])
                    elif line.strip().startswith("BLPport"):
                        if params['cemon.lrms'] == "pbs" and key.find('pbs') > -1:
                            outFile.write('BLPport="%s"\n' % params['pbs.log.parser.port'])
                        elif params['cemon.lrms'] == "lsf" and key.find('lsf') > -1:
                            outFile.write('BLPport="%s"\n' % params['lsf.log.parser.port'])
                    else:
                        outFile.write(line)
                        
                outFile.close()
                inFile.close()
                
                os.rename(key + ".new",key)

        print "Configuring the BLAHP log parser                             ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Workspace Service configuration
        #-------------------------------------------------------------------------------
        if self.verbose:
            print "Configuring the GT4 Workspace Service..."

        # Deploy service requires Ant
        if not os.environ.has_key('ANT_HOME'):
            print "Apache Ant doesn't seem to be correctly installed on this server. Please verify and re-run this script   ",
            glib.printFailedMessage()
            return 1
        if not os.path.exists(os.environ['ANT_HOME']):
            print "Apache Ant doesn't seem to be correctly installed on this server. Please verify and re-run this script   ",
            glib.printFailedMessage()
            return 1

        # Create globus user and set user rights
        uid,gid = glib.add_user("globus")

        os.system("/bin/chown -R globus:globus /opt/gt4")
      
        gt4_location = pwd.getpwnam("globus")[5] + "/globus/gt4"
        glib.check_dir(pwd.getpwnam("globus")[5] + "/globus")
        if os.path.exists(gt4_location):
            os.remove(gt4_location)
        os.symlink("/opt/gt4", gt4_location)
        wss_location = pwd.getpwnam("globus")[5] + "/workspaceService"
        if os.path.exists(wss_location):
            os.remove(wss_location)
        os.symlink("/opt/gt4", wss_location)

        # Set user env
        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('# Workspace service configuration') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('\n')
            f.write('# Workspace service configuration\n')
            f.close()
            
        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export JDK_HOME=${JAVA_HOME}') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export JDK_HOME=${JAVA_HOME}\n')
            f.close()
            
        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export JAVAC=${JAVA_HOME}/bin/javac') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export JAVAC=${JAVA_HOME}/bin/javac\n')
            f.close()
            
        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export CLASSPATH=.:${JAVA_HOME}/jre/lib/rt.jar:${JAVA_HOME}/lib/tools.jar') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export CLASSPATH=.:${JAVA_HOME}/jre/lib/rt.jar:${JAVA_HOME}/lib/tools.jar\n')
            f.close()
            
        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export LD_LIBRARY_PATH=${JAVA_HOME}/jre/lib/i386:${LD_LIBRARY_PATH}') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export LD_LIBRARY_PATH=${JAVA_HOME}/jre/lib/i386:${LD_LIBRARY_PATH}\n')
            f.close()
            
        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('unset GPT_LOCATION') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('unset GPT_LOCATION\n')
            f.close()
            
        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export GLOBUS_LOCATION=${HOME}/globus/gt4') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export GLOBUS_LOCATION=${HOME}/globus/gt4\n')
            f.close()

        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export WORKSPACE_HOME=${HOME}/workspaceService') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export WORKSPACE_HOME=${HOME}/workspaceService\n')
            f.close()

        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.startswith('export ANT_HOME='):
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export ANT_HOME=%s\n' % os.environ['ANT_HOME'])
            f.close()

        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export PATH=$ANT_HOME/bin:$PATH') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export PATH=$ANT_HOME/bin:$PATH\n')
            f.close()

        f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'r')
        for line in f:
            if line.find('export GLOBUS_OPTIONS=-Djava.security.egd=file:/dev/urandom') != -1:
                f.close()
                break
        else:
            f.close()
            f = open(pwd.getpwnam("globus")[5] + "/.bashrc", 'a')
            f.write('export GLOBUS_OPTIONS=-Djava.security.egd=file:/dev/urandom\n')
            f.close()

        # Copy certificates
        if self.verbose:
            print "Copying host certificates to GT4 container certificates"
        os.system("cp %s %s/containerkey.pem" % (params['host.key.file'], os.path.dirname(params['host.key.file'])))
        os.chown("%s/containerkey.pem" % os.path.dirname(params['host.key.file']), uid, gid)
        os.system("cp %s %s/containercert.pem" % (params['host.certificate.file'], os.path.dirname(params['host.certificate.file'])))
        os.chown("%s/containercert.pem" % os.path.dirname(params['host.certificate.file']), uid, gid)

        # Customize deploy-jndi-config.xml
        jndiconf = "%s/workspaceService/service/java/source/deploy-jndi-config.xml" % pwd.getpwnam("globus")[5]

        if os.path.exists(jndiconf + ".1"):
            os.remove(jndiconf + ".1")

        fin = open(jndiconf,'r')
        fout = open(jndiconf + ".1",'w')
        host = 0
        ttl = 0
        for line in fin:
            if host:
                fout.write("                    <value>%s</value>\n" % os.environ['HOSTNAME'])
                host = 0
            elif ttl:
                fout.write("                    <value>1440</value>\n")
                ttl = 0
            else:
                fout.write(line)

            if line.find('<name>host</name>') != -1:
                host = 1
            if line.find('<name>defaultTtl</name>') != -1:
                ttl = 1

        fin.close()
        fout.close()
        os.rename(jndiconf + ".1", jndiconf)

        # Deploy WSS web service and compile LCMAPS support for WSS
        fsh = open("%s/globus/wss-deploy.sh" % pwd.getpwnam("globus")[5],'w')
        fsh.write("#! /bin/bash\n")
        fsh.write("\n")
        fsh.write(". $GLOBUS_LOCATION/etc/globus-devel-env.sh\n")
        fsh.write("cd $WORKSPACE_HOME\n")
        fsh.write("ant deploy\n")
        fsh.write("mkdir -p $GLOBUS_LOCATION/var/log\n")
        fsh.write("cd $WORKSPACE_HOME/local/lcmaps/source\n")
        fsh.write("make install\n")
        fsh.close()

        # Execute the script as globus and then remove it
        os.chmod("%s/globus/wss-deploy.sh" % pwd.getpwnam("globus")[5],0755)
        os.system('su -l globus -c "%s/globus/wss-deploy.sh"' % pwd.getpwnam("globus")[5])
        os.remove("%s/globus/wss-deploy.sh" % pwd.getpwnam("globus")[5])

        # Set VO identity (it only works with one VO for the moment)
        os.system("cp %s/workspaceService/local/pooled/admin/expire_pooluser.sh %s/globus/gt4/bin" \
            % (pwd.getpwnam("globus")[5],pwd.getpwnam("globus")[5]))
        
        fin = open("%s/globus/gt4/bin/expire_pooluser.sh" % pwd.getpwnam("globus")[5], 'r')
        fout = open("%s/globus/gt4/bin/expire_pooluser.sh.1" % pwd.getpwnam("globus")[5], 'w')
        for line in fin:
            if line.find('POOL_PREFIX=') != -1:
                fout.write('POOL_PREFIX="%s"\n' % params['pool.account.basename'][0])
            elif line.find('POOL_PREFIX_STRLEN=') != -1:
                fout.write('POOL_PREFIX_STRLEN=%d\n' % len(params['pool.account.basename'][0]))
            elif line.find("DEFAULT_GROUP=") != -1:
                fout.write('DEFAULT_GROUP="%s"\n' % params['pool.account.group'][0])
            else:
                fout.write(line)

        fout.close()
        fin.close()

        os.rename("%s/globus/gt4/bin/expire_pooluser.sh.1" % pwd.getpwnam("globus")[5], \
            "%s/globus/gt4/bin/expire_pooluser.sh" % pwd.getpwnam("globus")[5])
        

        # Set log4j properties
        f = open("%s/globus/gt4/container-log4j.properties" % pwd.getpwnam("globus")[5], 'r')
        for line in f:
            print line
            if line.strip().startswith('log4j.category.org.globus.dynamicAccounts=INFO'):
                f.close()
                break
        else:
            f.close()
            f = open("%s/globus/gt4/container-log4j.properties" % pwd.getpwnam("globus")[5], 'a')
            f.write('\n# Workspace Service logging\n')
            f.write('log4j.category.org.globus.dynamicAccounts=INFO\n')
            f.close()

        # Create default ACL files
        file = open('%s/workspaceService/service/java/source/etc/attr-authz' \
            % pwd.getpwnam("globus")[5], 'w')
        file.write('/%s/Role=NULL/Capability=NULL\n' % params['voms.voname'][0])
        file.close()

        file = open('%s/workspaceService/service/java/source/etc/dn-authz' \
            % pwd.getpwnam("globus")[5], 'w')
        file.write('#/DC=org/DC=domain/OU=somecontainer/CN=Some Person 00000\n')
        file.close()
        
        # Set various files permissions to allow globus to access the file
        uid,gid = glib.get_user_info("globus")

        os.chown("%s/globus/gt4/bin/lcmaps_poolindex" % pwd.getpwnam("globus")[5],0,gid)
        os.chmod("%s/globus/gt4/bin/lcmaps_poolindex" % pwd.getpwnam("globus")[5],04750)
        
        os.chown(params['host.gridmap.dir'],0,gid)
        os.chmod(params['host.gridmap.dir'],0770)
        
        os.chown("%s/globus/gt4/bin/expire_pooluser.sh" % pwd.getpwnam("globus")[5], 0, gid)
        os.chmod("%s/globus/gt4/bin/expire_pooluser.sh" % pwd.getpwnam("globus")[5], 04750)
        
        os.chown('%s/workspaceService/service/java/source/etc/attr-authz' % pwd.getpwnam("globus")[5], 0, gid)
        os.chmod('%s/workspaceService/service/java/source/etc/attr-authz' % pwd.getpwnam("globus")[5], 0770)

        os.chown('%s/workspaceService/service/java/source/etc/dn-authz' % pwd.getpwnam("globus")[5], 0, gid)
        os.chmod('%s/workspaceService/service/java/source/etc/dn-authz' % pwd.getpwnam("globus")[5], 0770)

        # Clean some leftovers
        os.remove('/opt/globus/libexec/globus-sh-tools-vars.sh')
        os.remove('globus-script-initializer')
        os.remove('config.status')
        os.remove('config.log')
        
        #-------------------------------------------------------------------------------
        # Logger configuration
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring the Logging and Bookkeeping local logger daemon..."
    
        uid,gid = glib.get_user_info(os.environ['GLITE_USER'])
        
        # Check certificate directory, create if not existing
        if self.verbose:
            print "Checking user certificates directory %s/%s" % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path'])
        glib.check_dir("%s/%s" % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']),0755, uid, gid)
        
        # Copy certificates
        if self.verbose:
            print "Copying host certificates to logger user certificates directory"
        os.system("cp %s %s ~%s/%s" % (params['host.certificate.file'], params['host.key.file'], \
            os.environ['GLITE_USER'], params['user.certificate.path']))
        os.chown("%s/%s/hostcert.pem" % (pwd.getpwnam(os.environ['GLITE_USER'])[5], \
            params['user.certificate.path']), uid, gid)
        os.chown("%s/%s/hostkey.pem" % (pwd.getpwnam(os.environ['GLITE_USER'])[5], \
            params['user.certificate.path']), uid, gid)
        
        # Set files mode
        os.chmod("%s/bin/pbs_submit.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/pbs_status.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/pbs_cancel.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/lsf_submit.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/lsf_status.sh" % os.environ['GLITE_LOCATION'],0755)
        os.chmod("%s/bin/lsf_cancel.sh" % os.environ['GLITE_LOCATION'],0755)
        
        print "Configuring the Logging and Bookkeeping local logger daemon  ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Configure the CE Monitor
        #-------------------------------------------------------------------------------
        
        tomcat.stop()
        
        if self.verbose:
            print "Deploying the CE Monitor web service and plug-ins..."
        
        if os.path.exists("%s/webapps/ce-monitor.war" % os.environ['CATALINA_HOME']):
            if self.verbose:
                print "Removing existing CE monitor war file..."
            os.remove("%s/webapps/ce-monitor.war" % os.environ['CATALINA_HOME'])
            
        if os.path.exists("%s/webapps/ce-monitor" % os.environ['CATALINA_HOME']):
            if self.verbose:
                print "Removing existing CE monitor files..."
            os.system("rm -rf %s/webapps/ce-monitor" % os.environ['CATALINA_HOME'])

        os.system("cp %s/etc/glite-ce-monitor/ce-monitor.xml %s/conf/Catalina/localhost" \
            % (os.environ["GLITE_LOCATION"], os.environ["CATALINA_HOME"]))
            
        # Check war file
        if not params.has_key('ce-monitor.DOCBASE'):
            params['ce-monitor.DOCBASE'] = "%s/share/webapps/ce-monitor.war" % os.environ['GLITE_LOCATION']

        if not os.path.exists(params['ce-monitor.DOCBASE']):
            glib.printErrorMessage("The CE Monitor war file does not exist")
            glib.printErrorMessage("Configuration cannot continue")
            print "Deploying the CE Monitor web service and plug-ins         ",
            glib.printFailedMessage()
            return 1

        # Customize context file
        inputfile = '%s/conf/Catalina/localhost/ce-monitor.xml' % os.environ['CATALINA_HOME']
        doc = xml.dom.minidom.parse(inputfile)
        nodes = xpath.Evaluate("/Context", doc.documentElement)
        nodes[0].setAttribute('docBase',params['ce-monitor.DOCBASE'])
        nodes = xpath.Evaluate('/Context/Environment[@name="AuthZConfFileName"]', doc.documentElement)
        nodes[0].setAttribute('value',"%s/etc/glite-ce-monitor/authconfig.xml" % os.environ['GLITE_LOCATION'])
        nodes = xpath.Evaluate('/Context/Environment[@name="sslCertFile"]', doc.documentElement)
        nodes[0].setAttribute('value',"%s/%s/hostcert.pem" % (pwd.getpwnam("tomcat4")[5], \
            params['user.certificate.path']))
        nodes = xpath.Evaluate('/Context/Environment[@name="sslKey"]', doc.documentElement)
        nodes[0].setAttribute('value',"%s/%s/hostkey.pem" % (pwd.getpwnam("tomcat4")[5], \
            params['user.certificate.path']))
        nodes = xpath.Evaluate('/Context/Logger[@className="org.apache.catalina.logger.FileLogger"]', doc.documentElement)
        nodes[0].setAttribute('prefix',"glite-ce-monitor-")
        
        try:        
            outputfile = '%s/conf/Catalina/localhost/ce-monitor.xml' % os.environ['CATALINA_HOME']
            output = open(outputfile, "w")
            PrettyPrint(doc, stream=output)
            output.close()
        except IOError, ex:
            if ex.strerror == 'Permission denied':
                glib.printErrorMessage("File is not writable")
            print "Deploying the CE Monitor web service and plug-ins         ",
            glib.printFailedMessage()
                
        doc.unlink()
        
        # Change owner to tomcat5 user
        uid,gid = glib.get_user_info('tomcat4')
        os.system("/bin/chown -R %s:%s %s/conf/Catalina/localhost/ce-monitor.xml" % (uid,gid,os.environ['CATALINA_HOME']))
        
        # Give tomcat a chance to expand the war file
        tomcat.start()
        print "Waiting for Tomcat to expand the war file..."
        while not os.path.exists("%s/webapps/ce-monitor/config/sensorConfig.xml" % os.environ['CATALINA_HOME']):
            time.sleep(1)
        tomcat.stop()
        
#        # Update sensorConfig.xml with the correct plugin jarPath
#        inputfile = '%s/webapps/ce-monitor/config/sensorConfig.xml' % os.environ['CATALINA_HOME']
#        doc = xml.dom.minidom.parse(inputfile)
#        
#        NSS = {'ns1': 'http://glite.org/ce/monitorapij/types'}
#        ctx = xpath.Context.Context(doc, 1, 1, processorNss=NSS)
#        
#        nodes = xpath.Evaluate("/CEMonResources/ns1:CEMonResource/ns1:jarPath", context=ctx)
#        
#        print nodes[0].childNodes[0].data
#        nodes[0].childNodes[0].data = "file:%s/share/java/glite-ce-ce-plugin.jar" % os.environ['GLITE_LOCATION']
#        print nodes[0].childNodes[0].data
#        
#        outputfile = '%s/webapps/ce-monitor/config/sensorConfig.xml' % os.environ['CATALINA_HOME']
#        output = open(outputfile, "w")
#        PrettyPrint(doc, stream=output)
#        output.close()
#        
#        doc.unlink
        
        os.symlink("%s/share/java/glite-ce-ce-plugin.jar" % os.environ['GLITE_LOCATION'], \
                "%s/webapps/ce-monitor/sensors/CESensor.jar" % os.environ['CATALINA_HOME'])

        print "\nDeploying the CE Monitor web service and plug-ins          ",
        glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Configure the CE Plugin
        #-------------------------------------------------------------------------------
    
        if self.verbose:
            print "Configuring the CE subscription file..."

        # tomcat.restart()
        
        # Creating the predefined subscription file
        # Subscription expiration time is about six months from now, this is reset everytime the script is run
        expTime = time.strftime("%Y-%m-%dT%H:%M:%SZ",time.localtime(time.time() + 182*24*60*60))
        #subsfile = "%s/webapps/ce-monitor/subscriptions/predefinedSubscriptionList.xml" \
        #    % os.environ['CATALINA_HOME']
        subsfile = "%s/etc/glite-ce-monitor/predefinedSubscriptionList.xml" \
            % os.environ['GLITE_LOCATION']
        if self.verbose:
            print "Creating the predefined subscriptions file %s..." % subsfile
        file = open(subsfile,'w')
        file.write('<?xml version="1.0" encoding="utf-8"?>\n')
        file.write('<ns1:subscriptions xmlns:ns1="http://glite.org/ce/monitorapij/types">\n')
        file.write('\n')
        if params.has_key('cemon.wms.host') and params.has_key('cemon.wms.port'):
            
            if len(params['cemon.wms.host']) != len(params['cemon.wms.port']):
                glib.printErrorMessage("The 'cemon.wms.host' and 'cemon.wms.port' parameters must have the same size")
                glib.printErrorMessage("Configuration cannot continue")
                return 1
                
            for idx in xrange(len(params['cemon.wms.host'])):
                if params['cemon.wms.host'][idx].strip() != "":
                    file.write('    <ns1:Subscription xmlns:ns1="http://glite.org/ce/monitorapij/types" retryCount="-1">\n')
                    file.write('        <ns1:MonitorConsumerURL>http://%s:%s</ns1:MonitorConsumerURL>\n' % (params['cemon.wms.host'][idx], params['cemon.wms.port'][idx]))
                    file.write('        <ns1:Topic>\n')
                    file.write('            <ns1:Name>CE_MONITOR</ns1:Name>\n')
                    file.write('            <ns1:Dialect>\n')
                    file.write('                <ns1:Name>ISM_CLASSAD</ns1:Name>\n')
                    file.write('            </ns1:Dialect>\n')
                    file.write('        </ns1:Topic>\n')
                    file.write('        <ns1:Policy>\n')
                    file.write('            <ns1:rate>50000</ns1:rate>\n')
                    file.write('            <ns1:Query>\n')
                    file.write('                <ns1:Expression><![CDATA[%s]]></ns1:Expression>\n' % params['notifications.condition'])
                    file.write('                <ns1:QueryLanguage>ClassAd</ns1:QueryLanguage>\n')
                    file.write('            </ns1:Query>\n')
                    file.write('            <ns1:Action>\n')
                    file.write('                <ns1:doActionWhenQueryIs>true</ns1:doActionWhenQueryIs>\n')
                    file.write('                <ns1:name>SendNotification</ns1:name>\n')
                    file.write('            </ns1:Action>\n')
                    file.write('            <ns1:Action>\n')
                    file.write('                <ns1:doActionWhenQueryIs>false</ns1:doActionWhenQueryIs>\n')
                    file.write('                <ns1:name>SendExpiredNotification</ns1:name>\n')
                    file.write('            </ns1:Action>\n')
                    file.write('        </ns1:Policy>\n')
                    file.write('        <ns1:ExpirationTime>%s</ns1:ExpirationTime>\n' % expTime)
                    file.write('    </ns1:Subscription>\n')
                    file.write('\n')
        file.write('</ns1:subscriptions>\n')
        file.close()

        # Update wsdd file
        inputfile = '%s/webapps/ce-monitor/WEB-INF/server-config.wsdd' % os.environ['CATALINA_HOME']
        doc = xml.dom.minidom.parse(inputfile)
        
        nodes = xpath.Evaluate('//service[@name="CEMonitor"]', doc)
        if len(nodes) == 0:
            glib.printErrorMessage("Invalid service description file at %s" % inputfile)
            glib.printErrorMessage("Cannot continue")
            return 1
            
        newParam = doc.createElement('parameter')
        newParam.setAttribute('name','predefinedSubscriptionListPath')
        newParam.setAttribute('value',subsfile)
        nodes[0].appendChild(newParam)

        outputfile = inputfile
        output = open(outputfile, "w")
        PrettyPrint(doc, stream=output)
        output.close()
        
        doc.unlink
                
        if os.path.exists("%s/webapps/ce-monitor/subscriptions/predefinedSubscriptionList.xml" % os.environ['CATALINA_HOME']):
            os.remove("%s/webapps/ce-monitor/subscriptions/predefinedSubscriptionList.xml" % os.environ['CATALINA_HOME'])
        os.symlink("%s/etc/glite-ce-monitor/predefinedSubscriptionList.xml" % os.environ['GLITE_LOCATION'], \
                "%s/webapps/ce-monitor/subscriptions/predefinedSubscriptionList.xml" % os.environ['CATALINA_HOME'])

        # Generate the CE LCG Info Provider configuration file
        self.generateCEInfoProvidersFile()
        
        print "Configuring the CE subscription file                        ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------------------
        # Modify iptables
        #-------------------------------------------------------------------------------
    
        if self.iptables_config:
            
            if params.has_key('iptables.chain'):

                if self.verbose:
                    print "Opening required firewall ports..."
            
                if params['iptables.chain'] == "":
                    error_level = 2
                    glib.printWarningMessage("IPTables Chain not set, skipping firewall configuration")
                else:
                    
                    # Check if chain exists
                    if os.system("/sbin/iptables -L %s" % params['iptables.chain']) != 0:
                        if self.verbose:
                            print "Chain %s does not exist. Creating it..." % params['iptables.chain']
                        if os.system("/sbin/iptables -N %s" % params['iptables.chain']):
                            glib.printWarningMessage("An error occurred creating the new chain. Please configure the firewall manually")
                            error_level = 2
                        else:
                            if self.verbose:
                                print "Created chain %s" % params['iptables.chain']
                    else:
                        if self.verbose:
                            print "Using existing chain %s..." % params['iptables.chain']

                    # Configure the rules
                    rules = ["1 -m state --state NEW -m tcp -p tcp --dport 2119 -j ACCEPT",
                             "2 -m state --state NEW -m tcp -p tcp --dport 3878 -j ACCEPT",
                             "3 -m state --state NEW -m tcp -p tcp --dport 3879 -j ACCEPT",
                             "4 -m state --state NEW -m udp -p udp --dport 3879 -j ACCEPT",
                             "5 -m state --state NEW -m tcp -p tcp --dport 3882 -j ACCEPT",
                             "6 -m state --state NEW -m udp -p udp --dport 1020 -j ACCEPT",
                             "7 -m state --state NEW -m udp -p udp --dport 1021 -j ACCEPT",
                             "8 -m state --state NEW -m udp -p udp --dport 1022 -j ACCEPT",
                             "9 -m state --state NEW -m udp -p udp --dport 1023 -j ACCEPT",
                             "10 -m state --state NEW -m tcp -p tcp --dport 32768:65535 -j ACCEPT",
                             "11 -m state --state NEW -m udp -p udp --dport 32768:65535 -j ACCEPT",
                             "12 -m state --state NEW -m tcp -p tcp --dport 8443 -j ACCEPT"]
                    del_rules = ["ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:2119",
                                  "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:3878",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:3879",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:3879",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:3882",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1020",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1021",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1022",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpt:1023",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpts:32768:65535",
                                 "ACCEPT     udp  --  anywhere             anywhere           state NEW udp dpts:32768:65535",
                                 "ACCEPT     tcp  --  anywhere             anywhere           state NEW tcp dpt:8443"]

                    for idx in xrange(len(rules)):
                        # Delete rule if it exists
                        f = os.popen("/sbin/iptables -L %s" % params['iptables.chain'])
                        for line in f.xreadlines():
                            if line.find(del_rules[idx]) != -1:
                                if self.verbose:
                                    print "Rule '%s' already exist" % rules[idx]
                                break
                        else:
                            if os.system("/sbin/iptables -I %s %s" % (params['iptables.chain'],rules[idx])):
                                glib.printWarningMessage("An error occurred configuring a chain rule. Please configure the firewall manually")
                                error_level = 2
                                break
                            else:
                                if self.verbose:
                                    print "Added rule '%s'" % rules[idx]

                    if os.system('/etc/rc.d/init.d/iptables save'):
                        glib.printWarningMessage("An error occurred saving the iptables chain. Please configure the firewall manually")
                        error_level = 2
                
                print "Opening required firewall ports                              ",
                glib.printOkMessage()

        #-------------------------------------------------------------------
        # R-GMA servicetool: configure servicetool
        #-------------------------------------------------------------------

        if self.verbose:
            print "Configuring the R-GMA Service Publisher... "
            
        if rgmaServicetool.configureWithVersion(glib, self.version):
            # error in configuring services
            print "Configuring the R-GMA Service Publisher                      ",                                                
            glib.printFailedMessage()
            return 1
            
        print "Configuring the R-GMA Service Publisher                      ",
        glib.printOkMessage()
    
        #-------------------------------------------------------------------
        # R-GMA gin: configure R-GMA gadget-in tool
        #-------------------------------------------------------------------

        if self.verbose:
            print "Configuring R-GMA GIn ..."
        
        errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --subservice --configure --cemon=%s" \
            % (os.environ['GLITE_LOCATION'],params['rgma.gin.run_ce_provider']))
        if errorcode:
            print "Configuring R-GMA GadgetIn                                   ",                                                
            glib.printFailedMessage()
            return 1
        else:
            print "Configuring R-GMA GadgetIn                                   ",
            glib.printOkMessage()        
    
        return error_level
    
    #-------------------------------------------------------------------------------
    # Generate the CE LCG Info provider config files
    #-------------------------------------------------------------------------------

    def generateCEInfoProvidersFile(self):

        # Creating the CE plugin params file
        ceparamsfile = "%s/etc/glite-ce-ce-plugin/lcg-info-generic.conf" \
            % os.environ['GLITE_LOCATION']
        ceparamsfile1 = "%s/etc/glite-ce-ce-plugin/lcg-info-generic.conf.1" \
            % os.environ['GLITE_LOCATION']
        ceparamsfile_tmp = "%s/etc/glite-ce-ce-plugin/lcg-info-generic.conf.tmp" \
            % os.environ['GLITE_LOCATION']
        if os.path.exists(ceparamsfile1):
            os.remove(ceparamsfile1)
        if os.path.exists(ceparamsfile_tmp):
            os.remove(ceparamsfile_tmp)
        if os.path.exists(ceparamsfile):
            if self.verbose:
                print "Backing up existing file to %s..." % ceparamsfile1
            os.system("cp %s %s" % (ceparamsfile,ceparamsfile1))
 
        # Set queue type
        if not params.has_key('cemon.cetype'):
            params['cemon.cetype'] = "blah"
        else:
            if params['cemon.cetype'] == "":
                params['cemon.cetype'] = "blah"

        # Create file
        if self.verbose:
            print "Creating CE plugin LCG infor provider configuration file %s..." % ceparamsfile
        f = open(ceparamsfile_tmp, 'w')
        f.write("ldif_file=%s/etc/glite-ce-ce-plugin/out.ldif\n" % os.environ['GLITE_LOCATION'])
        f.write("generic_script=%s/libexec/lcg-info-generic\n" % params['lcg.providers.location'])
        f.write("wrapper_script=%s/etc/glite-ce-ce-plugin/lcg-info-wrapper\n" % os.environ['GLITE_LOCATION'])

        if params['cemon.lrms'] == "pbs":
            f.write("dynamic_script=%s/libexec/lcg-info-dynamic-%s %s/etc/glite-ce-ce-plugin/lcg-info-generic.conf %s\n" \
                % (params['lcg.providers.location'],\
                    params['cemon.lrms'],\
                    os.environ['GLITE_LOCATION'],\
                    os.environ['HOSTNAME']))
        elif params['cemon.lrms'] == "lsf":
            f.write("dynamic_script=%s/libexec/lcg-info-dynamic-%s %s %s/etc/glite-ce-ce-plugin/lcg-info-generic.conf\n" \
                % (params['lcg.providers.location'],\
                    params['cemon.lrms'],\
                    os.environ['LSF_BIN_PATH'],\
                    os.environ['GLITE_LOCATION']))
        else:
            f.write("dynamic_script=\n")
            
        f.write("temp_path=%s\n" % os.environ['GLITE_LOCATION_TMP'])
        f.write("template=%s/etc/GlueCE.template\n" % params['lcg.providers.location'])
        f.write("template=%s/etc/GlueCESEBind.template\n" % params['lcg.providers.location'])

        f.write("\n")
        f.write("# Common for all\n")
        f.write("GlueInformationServiceURL: undefined\n")

        f.write("\n")
        f.write("# CE Information Provider\n")

        # determine memorysize
        proclist = os.popen("free | /bin/grep ^Mem | /bin/awk '{print int($2/1024)}'")
        proc = proclist.readlines()
        for item in proc:
            memorysize = item.strip()
        proclist.close()

        f.write("\n")
        for vo in params['voms.voname']:
            f.write("GlueCEAccessControlBaseRule: VO:%s\n" % vo)
        f.write("GlueCEHostingCluster: %s\n" % params['cemon.cluster'])
        f.write("GlueCEInfoGatekeeperPort: %s\n" % params['gatekeeper.port'])
        f.write("GlueCEInfoHostName: %s\n" % os.environ['HOSTNAME'])
        f.write("GlueCEInfoLRMSType: %s\n" % params['cemon.lrms'])
        f.write("GlueCEInfoLRMSVersion: %s\n" % params['cemon.lrms.version'])
        f.write("GlueCEInfoTotalCPUs: 1\n")
        f.write("GlueCEPolicyMaxCPUTime: 172800\n")
        f.write("GlueCEPolicyMaxRunningJobs: 99999\n")
        f.write("GlueCEPolicyMaxTotalJobs: 999999\n")
        f.write("GlueCEPolicyMaxWallClockTime: 172800\n")
        f.write("GlueCEPolicyPriority: 1\n")
        f.write("GlueCEStateEstimatedResponseTime: 0\n")
        f.write("GlueCEStateFreeCPUs: 1\n")
        f.write("GlueCEStateRunningJobs: 0\n")
        f.write("GlueCEStateStatus: Production\n")
        f.write("GlueCEStateTotalJobs: 0\n")
        f.write("GlueCEStateWaitingJobs: 0\n")
        f.write("GlueCEStateWorstResponseTime: 0\n")
        f.write("GlueHostApplicationSoftwareRunTimeEnvironment: GLITE_1_2\n")
        f.write("GlueHostApplicationSoftwareRunTimeEnvironment: APP1\n")
        f.write("GlueHostApplicationSoftwareRunTimeEnvironment: APP2\n")
        f.write("GlueHostApplicationSoftwareRunTimeEnvironment: APP3\n")
        f.write("GlueHostApplicationSoftwareRunTimeEnvironment: APP4\n")
        f.write("GlueHostApplicationSoftwareRunTimeEnvironment: APP5\n")
        f.write("GlueHostArchitectureSMPSize: 2\n")
        f.write("GlueHostBenchmarkSF00: 380\n")
        f.write("GlueHostBenchmarkSI00: 400\n")
        f.write("GlueHostMainMemoryRAMSize: %s\n" % memorysize)
        f.write("GlueHostMainMemoryVirtualSize: %d\n" % (int(memorysize)*2))
        f.write("GlueHostNetworkAdapterInboundIP: FALSE\n")
        f.write("GlueHostNetworkAdapterOutboundIP: TRUE\n")
        f.write("GlueHostOperatingSystemName: SLC\n")
        f.write("GlueHostOperatingSystemRelease: 3.0.3\n")
        f.write("GlueHostOperatingSystemVersion: 3.0.3\n")
        f.write("GlueHostProcessorClockSpeed: 1000\n")
        f.write("GlueHostProcessorModel: PIII\n")
        f.write("GlueHostProcessorVendor: intel\n")

        f.write("\n")

        if params['cemon.cetype'] == "condorc":
            params['cemon.cetype'] = "blah"
             
        for queue in params['cemon.queues']:
            f.write("dn: GlueCEUniqueID=%s:%s/%s-%s-%s, mds-vo-name=local,o=grid\n" \
                % (os.environ['HOSTNAME'],\
                params['gatekeeper.port'],\
                params['cemon.cetype'],\
                params['cemon.lrms'],\
                queue))
            f.write("GlueCEName: %s\n" % queue)
            f.write("GlueForeignKey: GlueClusterUniqueID=%s\n" % params['cemon.cluster'])
            f.write("\n")

        f.write("\n")
        f.write("dn: GlueClusterUniqueID=%s, mds-vo-name=local,o=grid\n" % params['cemon.cluster'])
        f.write("GlueClusterService: %s\n" % params['cemon.cluster'])
        f.write("GlueClusterName: %s\n" % params['cemon.cluster'])
        for queue in params['cemon.queues']:
            f.write("GlueForeignKey: GlueCEUniqueID=%s:%s/%s-%s-%s\n" \
                % (os.environ['HOSTNAME'],\
                params['gatekeeper.port'],\
                params['cemon.cetype'],\
                params['cemon.lrms'],\
                queue))

        f.write("\n")
        f.write("dn: GlueSubClusterUniqueID=%s, GlueClusterUniqueID=%s, mds-vo-name=local,o=grid\n" \
            % (params['cemon.cluster'],params['cemon.cluster']))
        f.write("GlueChunkKey: GlueClusterUniqueID=%s\n" % params['cemon.cluster'])
        f.write("GlueSubClusterName: %s\n" % params['cemon.cluster'])

        queues = []
        seIds = []
        entrypoints = []
        for bind in params['cemon.cesebinds']:
            if bind != "":
                if bind.split()[0].strip().find("|") != -1:
                    queues.append(bind.split()[0].strip().split("|"))
                else:
                    queues.append(bind.split()[0].strip())
                seIds.append(bind.split()[1].strip())
                entrypoints.append(bind.split()[2].strip())
        
        f.write("\n")
        
        for q in params['cemon.queues']:
            f.write("dn: GlueCESEBindGroupCEUniqueID=%s:%s/%s-%s-%s, mds-vo-name=local,o=grid\n" \
                % (os.environ['HOSTNAME'],\
                params['gatekeeper.port'],\
                params['cemon.cetype'],\
                params['cemon.lrms'],\
                q))
            for idx in xrange(len(seIds)):
                if queues[idx] == "'.'" or queues[idx] == q:
                    f.write("GlueCESEBindGroupSEUniqueID: %s\n" % seIds[idx])
            f.write("\n")
            
        
        f.write("\n")
        
        for idxSE in xrange(len(seIds)):
            for idxq in xrange(len(params['cemon.queues'])):
                if queues[idxSE] == "'.'" or queues[idxSE] == params['cemon.queues'][idxq]:
                    f.write("dn: GlueCESEBindSEUniqueID=%s, GlueCESEBindGroupCEUniqueID=%s:%s/%s-%s-%s, mds-vo-name=local,o=grid\n" \
                        % (seIds[idxSE],\
                        os.environ['HOSTNAME'],\
                        params['gatekeeper.port'],\
                        params['cemon.cetype'],\
                        params['cemon.lrms'],\
                        params['cemon.queues'][idxq]))
                    f.write("GlueCESEBindCEAccesspoint: %s\n" % entrypoints[idxSE])
                    f.write("GlueCESEBindCEUniqueID: %s:%s/%s-%s-%s\n" \
                        % (os.environ['HOSTNAME'],\
                        params['gatekeeper.port'],\
                        params['cemon.cetype'],\
                        params['cemon.lrms'],\
                        params['cemon.queues'][idxq]))
            f.write("\n")
            
        f.close()
        os.rename(ceparamsfile_tmp,ceparamsfile)
         
        # Load configuration in LCG Info Providers
        os.system("%s/sbin/lcg-info-generic-config %s/etc/glite-ce-ce-plugin/lcg-info-generic.conf" \
            % (params['lcg.providers.location'],os.environ['GLITE_LOCATION']))

        uid,gid = glib.get_user_info('tomcat4')
        os.chown("%s/etc/glite-ce-ce-plugin/out.ldif" % os.environ['GLITE_LOCATION'], uid, gid)
        os.chmod("%s/etc/glite-ce-ce-plugin/out.ldif" % os.environ['GLITE_LOCATION'], 0644)
        os.chown("%s/etc/glite-ce-ce-plugin/lcg-info-wrapper" % os.environ['GLITE_LOCATION'], uid, gid)
        os.chmod("%s/etc/glite-ce-ce-plugin/lcg-info-wrapper" % os.environ['GLITE_LOCATION'], 0755)

        os.system("/bin/ln -fs %s/etc/glite-ce-ce-plugin/lcg-info-wrapper  %s/etc/glite-ce-ce-plugin/glite-ce-info" \
            % (os.environ['GLITE_LOCATION'],os.environ['GLITE_LOCATION']))

        print "Configuring CE Information provider                          ",
        glib.printOkMessage()
        
        return 0
            
    #-------------------------------------------------------------------------------
    # Start all services
    #-------------------------------------------------------------------------------

    def start(self):

        if len(os.listdir(params['X509_VOMS_DIR'])) == 0:
            print("\n\n")
            glib.printErrorMessage("No VOMS Server host certificates could be found in %s" % params['X509_VOMS_DIR'])
            glib.printErrorMessage("Before using the %s," % service.friendly_name)
            glib.printErrorMessage("make sure that the host certificate of each used VOMS")
            glib.printErrorMessage("Server is installed in the %s directory." % params['X509_VOMS_DIR'])
            return 1

        #-------------------------------------------------------------------------------
        # Start the gatekeeper
        #-------------------------------------------------------------------------------
    
        gatekeeper = '%s/sbin/glite-gatekeeper' % os.environ['GLITE_LOCATION']
        pid = glib.getPID('glite-gatekeeper')
    
        if pid != 0:
            glib.printInfoMessage('Gatekeeper is already running. Stopping and restarting the gLite Gatekeeper ...')
            os.system('kill -9 %d' % pid)
        else:
            print "Starting the gLite gatekeeper ... "
    
        os.system( "%s -conf %s/etc/gatekeeper.conf" % ( gatekeeper, os.environ['GLITE_LOCATION']) )
    
        # Check that the daemon is running
        pid = glib.getPID('glite-gatekeeper')
        if pid != 0:
            print "Starting the gLite Gatekeeper                                ",                                                
            glib.printOkMessage()
        else:
            print "Starting the gLite Gatekeeper                                ",                                                
            glib.printFailedMessage()
            return 1

        #-------------------------------------------------------------------------------
        # Start the locallogger
        #-------------------------------------------------------------------------------
    
        locallogger = "%s/etc/init.d/glite-lb-locallogger" % os.environ['GLITE_LOCATION']

        pid1 = glib.getPID('glite-lb-logd')
        pid2 = glib.getPID('glite-lb-interlogd')
        if (pid1 != 0) or (pid2 != 0):
            glib.printInfoMessage('LB Local Logger daemon already running. Stopping and restarting the LB Local Logger Daemons ...')
            os.system( "%s stop" % locallogger)
        else:
            print "Starting the gLite LB Local Logger ..."
    
        os.system( "%s start" % locallogger)
    
        # Check that the daemon is running
        pid1 = glib.getPID('glite-lb-logd')
        pid2 = glib.getPID('glite-lb-interlogd')
        if (pid1 != 0) and (pid2 != 0):
            print "The LB Local Logger was started                              ",                                                
            glib.printOkMessage()
            print "The LB Interlogger was started                               ",                                                
            glib.printOkMessage()
        else:
            glib.printErrorMessage("Could not start the LB Local Logger")
            glib.printErrorMessage("Please verify and re-run the script")
            return 1

        #-------------------------------------------------------------------------------
        # Start Tomcat
        #-------------------------------------------------------------------------------
    
        print "Starting Tomcat and the CE Monitor ..."
        pid = glib.getPID("tomcat")
        if pid != 0:
            glib.printInfoMessage("Tomcat is running already. Stopping and restarting Tomcat ...")
            tomcat.stop()
            
        tomcat.start()
    
        # Check that the daemon is running
        pid = glib.getPID("tomcat")
        if pid == 0:
            glib.printErrorMessage("Could not start Tomcat")
            glib.printErrorMessage("Please verify and re-run the script")
            return 1
        else:
            print "Tomcat has been started                                      ",                                                
            glib.printOkMessage()

        #-------------------------------------------------------------------
        # Start Servicetool
        #-------------------------------------------------------------------

        pid = glib.getPID('rgma-servicetool')
        if (pid != 0):
            glib.printInfoMessage('The gLite R-GMA servicetool is already running. Stopping and restarting ...')
            rgmaServicetool.stop()
            time.sleep(5)
        else:
            print "Starting the gLite R-GMA Servicetool service ..."

        errorcode = rgmaServicetool.start()
                                                                                                           
        if (errorcode == 0):
            print "The gLite R-GMA Service Publisher has been started           ",                                                
            glib.printOkMessage()
        else:
            glib.printErrorMessage("Could not start the gLite R-GMA Service Publisher")
            glib.printErrorMessage("Please verify and re-run the script")
            return 1
    
        #-------------------------------------------------------------------
        # Start R-GMA gin
        #-------------------------------------------------------------------

        print "Starting the gLite R-GMA GIN service ... "
                       
        errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --start --subservice" % os.environ['GLITE_LOCATION'])
        if errorcode:
            print "Starting the gLite R-GMA GIN service                         ",                                                
            glib.printFailedMessage()
            return 1

        print "Starting the gLite R-GMA GIN service                         ",                                                
        glib.printOkMessage()
        
        #-------------------------------------------------------------------
        # Start GT4 WS Container
        #-------------------------------------------------------------------

        # May not be configured yet, verfy and skip if required
        try:
            
            fsh = open("%s/globus/gt4-stop.sh" % pwd.getpwnam("globus")[5],'w')
            fsh.write("#! /bin/bash\n")
            fsh.write("\n")
            fsh.write("LD_LIBRARY_PATH=/opt/globus/lib:$LD_LIBRARY_PATH \\\n")
            fsh.write("%s/bin/grid-proxy-init -cert %s/containercert.pem \\\n" \
                % (os.environ['GLOBUS_LOCATION'],os.path.dirname(params['host.key.file'])))
            fsh.write("-key %s/containerkey.pem -valid 0:10\n" \
                % os.path.dirname(params['host.key.file']))
            fsh.write("$GLOBUS_LOCATION/bin/globus-stop-container")
            fsh.close()
            os.chmod("%s/globus/gt4-stop.sh" % pwd.getpwnam("globus")[5],0755)
    
            errorcode = os.system('su -l globus -c "%s/globus/gt4-stop.sh"' % pwd.getpwnam("globus")[5])
                
            if self.verbose:
                print "Starting the GT4 WS Container... "
                           
            fsh = open("%s/globus/gt4-start.sh" % pwd.getpwnam("globus")[5],'w')
            fsh.write("#! /bin/bash\n")
            fsh.write("\n")
            fsh.write("$GLOBUS_LOCATION/bin/globus-start-container \\\n")
            fsh.write("-p 8453 \\\n")
            fsh.write("-containerDesc $GLOBUS_LOCATION/etc/globus_wsrf_core/global_security_descriptor.xml \\\n")
            fsh.write("2>&1 > $GLOBUS_LOCATION/globus.log &\n")
            fsh.close()
            os.chmod("%s/globus/gt4-start.sh" % pwd.getpwnam("globus")[5],0755)
    
            errorcode = os.system('su -l globus -c "%s/globus/gt4-start.sh"' % pwd.getpwnam("globus")[5])
            if errorcode:
                print "Starting the GT4 WS Container                            ",                                                
                glib.printFailedMessage()
                return 1
    
            print "Starting the GT4 WS Container                             ",                                                
            glib.printOkMessage()

        except:
            glib.printWarningMessage("The WS container is not configured. Skipping for now....")
            
        return 0
    
    #-------------------------------------------------------------------------------
    # Stop all services
    #-------------------------------------------------------------------------------

    def stop(self):

        #-------------------------------------------------------------------
        # Stop Condor
        #-------------------------------------------------------------------
        condor.stop()

        #-------------------------------------------------------------------------------
        # Stop the gatekeeper
        #-------------------------------------------------------------------------------
    
        gatekeeper = '%s/sbin/glite-gatekeeper' % os.environ['GLITE_LOCATION']

        pid = glib.getPID('glite-gatekeeper')
        if pid != 0:
            print 'Stopping the gLite Gatekeeper...'
            os.system('kill -9 %d' % pid)
            pid = glib.getPID('glite-gatekeeper')
            if pid != 0:
                print "Stopping the gLite gatekeeper                                ",                                        
                glib.printFailedMessage()
            else:
                print "Stopping the gLite gatekeeper                                ",                                        
                glib.printOkMessage()
        else:
            print "The gLite gatekeeper was already stopped                     ",                                        
            glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Stop the local logger
        #-------------------------------------------------------------------------------
    
        locallogger = "%s/etc/init.d/glite-lb-locallogger" % os.environ['GLITE_LOCATION']

        pid1 = glib.getPID('glite-lb-logd')
        pid2 = glib.getPID('glite-lb-interlogd')

        if (pid1 != 0) or (pid2 != 0):
            print 'Stopping the gLite LB Local Logger ...'
            os.system( "%s stop" % locallogger)
            time.sleep(5)
            pid1 = glib.getPID('glite-lb-logd')
            pid2 = glib.getPID('glite-lb-interlogd')
            if (pid1 != 0) or (pid2 != 0):
                print "Stopping the LB Local Logger                                 ",                            
                glib.printFailedMessage()
            else:
                print "Stopping the LB Local Logger                                 ",                            
                glib.printOkMessage()
        else:
            print "The LB Local Logger was already stopped                      ",                            
            glib.printOkMessage()

        #-------------------------------------------------------------------------------
        # Stop Tomcat
        #-------------------------------------------------------------------------------
    
        pid = glib.getPID("tomcat")
        if pid != 0:
            print "Stopping Tomcat and the CE Monitor ..."
            tomcat.stop()
            time.sleep(5)
            pid = glib.getPID("tomcat")
            if pid != 0:
                print "Stopping Tomcat                                              ",                
                glib.printFailedMessage()
            else:
                print "Stopping Tomcat                                              ",                
                glib.printOkMessage()
        else:
            print "Tomcat was already stopped                                   ",                
            glib.printOkMessage()

        #-------------------------------------------------------------------
        # Stop the servicetool
        #-------------------------------------------------------------------
        pid = glib.getPID('rgma-servicetool')
        if (pid != 0):
            rgmaServicetool.stop()
            pid = glib.getPID('rgma-servicetool')
            if (pid != 0):
                print "Stopping R-GMA Service Publisher                             ",
                glib.printFailedMessage()
            else:
                print "Stopping R-GMA Service Publisher                             ",
                glib.printOkMessage()
        else:
            print "R-GMA Service Publisher was already stopped                  ",
            glib.printOkMessage()

        #-------------------------------------------------------------------
        # Stop R-GMA gin
        #-------------------------------------------------------------------

        print "Stopping the gLite R-GMA GIN service ... "
                       
        errorcode = os.system("%s/etc/config/scripts/glite-rgma-gin-config.py --stop --subservice" % os.environ['GLITE_LOCATION'])
        if errorcode:
            print "Stopping the gLite R-GMA GIN service                         ",
            glib.printFailedMessage()
            return 1
        
        print "Stopping the gLite R-GMA GIN service                         ",
        glib.printOkMessage()
            
        #-------------------------------------------------------------------
        # Stop the GT4 Container
        #-------------------------------------------------------------------

        if self.verbose:
            print "Stopping the GT4 WS Container... "

        try:
            fsh = open("%s/globus/gt4-stop.sh" % pwd.getpwnam("globus")[5],'w')
            fsh.write("#! /bin/bash\n")
            fsh.write("\n")
            fsh.write("LD_LIBRARY_PATH=/opt/globus/lib:$LD_LIBRARY_PATH \\\n")
            fsh.write("%s/bin/grid-proxy-init -cert %s/containercert.pem \\\n" \
                % (os.environ['GLOBUS_LOCATION'],os.path.dirname(params['host.key.file'])))
            fsh.write("-key %s/containerkey.pem -valid 0:10\n" \
                % os.path.dirname(params['host.key.file']))
            fsh.write("$GLOBUS_LOCATION/bin/globus-stop-container -s https://%s:8453/wsrf/services/ShutdownService" % glib.hostname)
            fsh.close()
            os.chmod("%s/globus/gt4-stop.sh" % pwd.getpwnam("globus")[5],0755)
    
            errorcode = os.system('su -l globus -c "%s/globus/gt4-stop.sh"' % pwd.getpwnam("globus")[5])
            if errorcode:
                print "Stopping the GT4 WS Container                          ",
                glib.printFailedMessage()
                return 1

        except:
            glib.printWarningMessage("The WS container is not configured. Skipping for now....")
            
        print "Stopping the the GT4 WS Container                        ",
        glib.printOkMessage()
            
        return 0
        
    #-------------------------------------------------------------------------------
    # Service status
    #-------------------------------------------------------------------------------

    def status(self):

        print "\nxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx"
        print "       Checking status of %s" % self.friendly_name
        print "xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx\n"

        error_level = 0
        
        #-------------------------------------------------------------------------------
        # Gatekeeper
        #-------------------------------------------------------------------------------
    
        gatekeeper = '%s/sbin/glite-gatekeeper' % os.environ['GLITE_LOCATION']
        pid = glib.getPID('glite-gatekeeper')
        if pid != 0:
            print "The gLite Gatekeeper is running"
        else:
            print "The gLite Gatekeeper is not running"
            error_level = 1

        #-------------------------------------------------------------------------------
        # Local logger
        #-------------------------------------------------------------------------------
    
        locallogger = "%s/etc/init.d/glite-lb-locallogger" % os.environ['GLITE_LOCATION']
        retval = os.system( "%s status" % locallogger)
        if retval != 0:
            error_level = 1

        #-------------------------------------------------------------------------------
        # CE Monitor
        #-------------------------------------------------------------------------------
    
        if params.has_key('host.certificate.file'):
            hostcert_file = params['host.certificate.file']
        else:
            hostcert_file = "/etc/grid-security/hostcert.pem"
        
        if params.has_key('host.key.file'):    
            hostkey_file  = params['host.key.file']
        else:
            hostkey_file = "/etc/grid-security/hostkey.pem"
        
        print "The CE Monitor is ",
        status = glib.checkHTTPSconnection("localhost", 
            "/ce-monitor/services/Version?wsdl",
            8443, hostkey_file, hostcert_file)
                                    
        if status[0]:
            print "not running."
            error_level = 1
        else:
            print "running."
                
        if error_level:
            print "\nThe %s is not running." % self.friendly_name
        else:
            print "\nThe %s is running." % self.friendly_name
            
        return error_level
        
#-------------------------------------------------------------------------------
# Set all environment variables
#-------------------------------------------------------------------------------

def set_env():

    # gLite
    glib.export('GLITE_LOCATION');
    glib.export('GLITE_LOCATION_VAR');
    if not os.path.exists(os.environ['GLITE_LOCATION_VAR']):
        os.mkdir(os.environ['GLITE_LOCATION_VAR'],0755)
    glib.export('GLITE_LOCATION_LOG');
    if not os.path.exists(os.environ['GLITE_LOCATION_LOG']):
        os.mkdir(os.environ['GLITE_LOCATION_LOG'],0755)
    glib.export('GLITE_LOCATION_TMP');
    if not os.path.exists(os.environ['GLITE_LOCATION_TMP']):
        os.mkdir(os.environ['GLITE_LOCATION_TMP'],0755)

    glib.export('CONDOR_CONFIG')
    glib.export('GLOBUS_LOCATION')
    glib.export('GPT_LOCATION')
    glib.export('CATALINA_HOME')
    glib.export('JAVA_HOME')

    if params['cemon.lrms'] == 'pbs':
        glib.export('PBS_BIN_PATH',params['cemon.cluster-batch-system-bin-path'])
        glib.export('PBS_SPOOL_DIR',params['PBS_SPOOL_DIR'])
    elif params['cemon.lrms'] == 'lsf':
        glib.export('LSF_BIN_PATH',params['cemon.cluster-batch-system-bin-path'])
        glib.export('LSF_CONF_PATH',params['LSF_CONF_PATH'])
        
    # Check user, create it if not existing
    glib.export('GLITE_USER',params['lb.user'])
    if glib.verbose:
        print "Checking local logger user"
    uid,gid = glib.add_user(os.environ['GLITE_USER'])
        
    glib.export('GLITE_HOST_CERT',"%s/%s/hostcert.pem" \
        % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']))
    glib.export('GLITE_HOST_KEY',"%s/%s/hostkey.pem" \
        % (pwd.getpwnam(os.environ['GLITE_USER'])[5],params['user.certificate.path']))

    if not params.has_key('X509_VOMS_DIR'):
        params['X509_VOMS_DIR'] = '/etc/grid-security/vomsdir'
    glib.export('X509_VOMS_DIR', params['X509_VOMS_DIR'])

    # bin and lib paths
    glib.addEnvPath("PATH","%s/bin:%s/bin:%s/bin:/opt/condor-c/libexec/glite:%s/externals/bin:%s/bin" \
        % (params['lcg.providers.location'],\
           os.environ['JAVA_HOME'],\
           os.environ['GLOBUS_LOCATION'],\
           os.environ['GLITE_LOCATION'],\
           os.environ['GLITE_LOCATION']))
    glib.addEnvPath("LD_LIBRARY_PATH","%s/lib:%s/lib:%s/externals/lib:%s/lib" \
        % (params['lcg.providers.location'],\
           os.environ['GLOBUS_LOCATION'],\
           os.environ['GLITE_LOCATION'],\
           os.environ['GLITE_LOCATION']))

    # MANPATH
    glib.addEnvPath('MANPATH', os.environ['GLOBUS_LOCATION'] + '/man')
    glib.addEnvPath('MANPATH', os.environ['GLITE_LOCATION'] + '/share/man')

    # Perl
    glib.addEnvPath("PERL5LIB", "%s/lib/perl:%s/lib/perl5" % (os.environ['GPT_LOCATION'],os.environ['GLITE_LOCATION']))

    # Set environment
    glib.setUserEnv()
    
#-------------------------------------------------------------------------------
#  Main program begins here 
#-------------------------------------------------------------------------------

if __name__ == '__main__':

    # The script must be run as root
    if not os.geteuid() == 0:
        print '"\nThis script must be run as root\n'
        sys.exit(1)
        
    # Get an instance of the library class
    glib = gLib()
    glib.verbose = 0
    
    # Load parameters
    params = {}
    try:
        opts, args = glib.getopt(sys.argv[1:], '', ['siteconfig='])
        for o, a in opts:
            if o == "--siteconfig":
                params['site.config.url'] = a
                break
    except getopt.GetoptError:
        pass
    if glib.loadConfiguration("%s/../glite-ce.cfg.xml" % glib.getScriptPath(),params):
        print "An error occurred while configuring the service"
        sys.exit(1)
    
    verbose = 0
    if params.has_key('glite.installer.verbose'):
        if params['glite.installer.verbose'] == "true":
            verbose = 1
    glib.verbose = verbose
    
    # Set up the environment
    set_env()
          
    # Instantiate the service classes
    service = glite_ce()
    service.verbose = verbose
    
    # Instantiate the condor class
    condor = Condor(params)
    condor.verbose = verbose
    
    # Instantiate the globus class
    globus = Globus(params)
    globus.verbose = verbose
    
    # Instantiate the tomcat class
    tomcat = Tomcat5(params)
    tomcat.verbose = verbose
    
    # Instantiate the rgma servicetool class
    rgmaServicetool = gliteRgmaServicetool()
    rgmaServicetool.verbose = verbose
    
    # Command line opts if any
    try:
        opts, args = glib.getopt(sys.argv[1:], 'nchv', ['noiptables', 'checkconf', 'help', 'version', 'configure', 'start', 'stop', 'status', 'siteconfig='])
    except getopt.GetoptError:
        service.usage(msg = "Unknown options(s)")
        sys.exit(1)

    if len(opts) == 0:
        service.usage()
        sys.exit(0)
    
    # Check cli options
    for o, a in opts:

        if o in ("-n", "--noiptables"):
            service.iptables_config = 0

        if o in ("-h", "--help"):
            service.usage()
            sys.exit(0)

        if o in ("-v", "--version"):
            service.showVersion()
            sys.exit(0)

        if o in ("-c", "--checkconf"):
            service.copyright()
            service.showVersion()
            glib.print_params(params)
            sys.exit(0)

        if o == "--configure":

            # Check certificates
            if params.has_key('glite.installer.checkcerts'):
                if params['glite.installer.checkcerts'] == "true":
                    if glib.check_certs(params) != 0:
                        print "An error occurred while configuring the %s" \
                            % service.friendly_name
                        sys.exit(1)
            
            service.copyright()
            service.showVersion()
            service.banner()
                
            # Display the parameters if verbose output
            if verbose:
                glib.print_params(params)
                
            # Stop all services
            glib.printInfoMessage("\n\nStopping all running CE services...")
            service.stop()
            
            # Configure the service
            return_value = service.configure()
            
            if return_value == 0:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running CE services...")
                service.stop()
                
                glib.registerService()

                print "\n\nThe %s configuration was successfully completed\n" % service.friendly_name
                print "\nBefore using the %s services make sure that" % service.friendly_name
                print "the host certificate of each used VOMS server is installed in"
                print "%s\n" % os.environ['X509_VOMS_DIR']
                print "You can now start the service using the --start option of this script\n\n"
    
                sys.exit(0)

            elif return_value == 2:

                # Stop all services
                glib.printInfoMessage("\n\nStopping all running WMS services...")
                service.stop()
                
                print "\n\nThe %s configuration was completed, but one or more warnings" % service.friendly_name
                print "were issued. Please revise them and re-run the script"
                print "or configure and start the %s manually\n" % service.friendly_name

                sys.exit(2)

            else:
                print "\n\nAn unrecoverable error occurred while configuring the %s service" \
                    % service.friendly_name

                sys.exit(1)
            
        if o == "--start":
            if service.start() == 0:
                print "\n\nThe %s was successfully started\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while starting the %s service" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--stop":
            if service.stop() == 0:
                print "\n\nThe %s was successfully stopped\n" % service.friendly_name
                sys.exit(0)
            else:
                print "\n\nAn unrecoverable error occurred while stopping the %s service" \
                    % service.friendly_name
                sys.exit(1)

        if o == "--status":
            sys.exit(service.status())
